import re

class Integer(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, unuse, unuse_):
        if value == None or value == '':
            return
        if not re.match("^-?[0-9]+$", str(value)):
            return 'NOT_INTEGER'

class PositiveInteger(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, unuse, unuse_):
        if value == None or value == '':
            return
        if not re.match("^[1-9][0-9]*$", str(value)):
            return 'NOT_POSITIVE_INTEGER'

class Decimal(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, unuse, unuse_):
        if value == None or value == '':
            return
        if not re.match("^(?:\-?(?:[0-9]+\.[0-9]+)|(?:[0-9]+))$", str(value)):
            return 'NOT_DECIMAL'

class PositiveDecimal(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, unuse, unuse_):
        if value == None or value == '':
            return
        if not re.match("^(?:(?:[0-9]*\.[0-9]+)|(?:[1-9][0-9]*))$", str(value)):
            return 'NOT_POSITIVE_DECIMAL'

class MaxNumber(object):
    def __init__(self, *args):
        self._max_number = float(args[1])

    def __call__(self, number, unuse, unuse_):
        if number == None or number == '':
            return

        if float(number) > self._max_number:
            return 'TOO_HIGH'

class MinNumber(object):
    def __init__(self, *args):
        self._min_number = float(args[1])

    def __call__(self, number, unuse, unuse_):
        if number == None or number == '':
            return

        if float(number) < self._min_number:
            return 'TOO_LOW'

class BetweenNumbers(object):
    def __init__(self, *args):
        self._min_number = float(args[1])
        self._max_number = float(args[2])

    def __call__(self, number, unuse, unuse_):
        if number == None or number == '':
            return

        if float(number) > self._max_number:
            return 'TOO_HIGH'
        if float(number) < self._min_number:
            return 'TOO_LOW'



import os
import sys
PATH_TO_RULES = '/'.join(str(os.path.abspath(__file__)).split('/')[:-1])+'/Rules'
sys.path.insert(0,PATH_TO_RULES)

import Common
import Filters
import Helper
import Special
import String
import Numeric

DEFAULT_RULES = {
    'required':         Common.Required,
    'not_empty':        Common.NotEmpty,
    'not_empty_list':   Common.NotEmptyList,

    'one_of':           String.OneOf,
    'max_length':       String.MaxLength,
    'min_length':       String.MinLength,
    'length_equal':     String.EqualLength,
    'length_between':   String.BetweenLength,
    'like':             String.Like,

    'integer':          Numeric.Integer,
    'positive_integer': Numeric.PositiveInteger,
    'decimal':          Numeric.Decimal,
    'positive_decimal': Numeric.PositiveDecimal,
    'max_number':       Numeric.MaxNumber,
    'min_number':       Numeric.MinNumber,
    'number_between':   Numeric.BetweenNumbers,

    'email':            Special.Email,
    'equal_to_field':   Special.EqualToField,
    'url':              Special.Url,
    'iso_date':         Special.IsoDate,

    'nested_object':    Helper.NestedObject,
    'list_of':          Helper.ListOf,
    'list_of_objects':  Helper.ListOfObjects,
    'list_of_different_objects': Helper.ListOfDiferentObjects,

    'trim':             Filters.Trim,
    'to_lc':            Filters.ToLc,
    'to_uc':            Filters.ToUc,
    'remove':           Filters.Remove,
    'leave_only':       Filters.LeaveOnly,

}




import BuildAliasedRule
from DEFAULT_RULES import DEFAULT_RULES
IS_DEFAULT_AUTO_TRIM = False;

class Validator(object):
    def __init__(self, livr_rules, is_auto_trim = None):
        self.__is_prepare = False
        self.__livr_rules = livr_rules
        self.__validators = {}
        self.__validator_builders = {}
        self.__errors = None

        if is_auto_trim == None:
            self.__is_auto_trim = IS_DEFAULT_AUTO_TRIM
        else:
            self.__is_auto_trim = is_auto_trim
        self.register_rules(DEFAULT_RULES)
    
    @staticmethod
    def register_default_rules(rules):
        for name, rule in rules.items():
            DEFAULT_RULES[name] = rule

    @staticmethod
    def register_aliased_default_rule(alias):
        BuildAliasedRule.BuildAliasedRule(DEFAULT_RULES, alias)

    @staticmethod
    def set_default_auto_trim(is_auto_trim):
        IS_DEFAULT_AUTO_TRIM = bool(is_auto_trim)

    def register_aliased_rule(self, alias):
        BuildAliasedRule.BuildAliasedRule(self.__validator_builders, alias)

    def __make_validators(self, rules):
        if isinstance(rules, dict):
            return [self.__build_validator(**self.__parse_rule({name:rule})) for name, rule in rules.items()]
        if isinstance(rules, list):
            return [self.__build_validator(**self.__parse_rule(rule)) for rule in rules]
        
        return [self.__build_validator(**self.__parse_rule(rules))]

    def prepare(self):
        if self.__is_prepare:
            return

        for name, rules in self.__livr_rules.items():
            self.__validators[name] = self.__make_validators(rules)
        
        self.__is_prepare = True
    
    def validate(self, data):
        if not self.__is_prepare:
            self.prepare()
        if self.__is_auto_trim:
            self.__auto_trim(data)
        if not isinstance(data, dict):
            self.__error = "FORMAT_ERROR"
            return
        
        errors  = {}
        result = {}

        for field_name, validators in self.__validators.items():
            if not validators:
                continue

            mid_result = []
            value = data[field_name] if field_name in data else None
            
            for func in validators:
                arg = result[field_name] if field_name in result else value
                error_code = func(arg, data, mid_result)

                if error_code:
                    errors[field_name] = error_code
                    break
                elif value != None:
                    result[field_name] = mid_result[0] if len(mid_result) else value

        if not errors:
            self.__errors = None
            return result
        else:
            self.__errors = errors
            return False

    def get_errors(self):
        return self.__errors

    def get_rules(self):
        return self.__validator_builders

    def register_rules(self, rules):
        for rule_name, rule_value in rules.items():
            self.__validator_builders[rule_name] = rule_value

    def __parse_rule(self,  livr_rule):
        if isinstance(livr_rule, dict):
            name = list(livr_rule.keys())[0]
            content = livr_rule[name]

            if not isinstance(content, list):
                content = [content]
        else:
            name = livr_rule
            content = []
        return {"name": name, "args": content}

    def __build_validator(self, name, args):
        if not name in self.__validator_builders:
            raise Exception("Rule [{}] not registered".format(name))
        return self.__validator_builders[name](self.__validator_builders, *args)

    def __auto_trim(self, data):
        if type(data) is str:
            return data.strip()
        
        elif type(data) is list:
            trimmed_list = []
            
            for val in data:
                trimmed_list.append(self.__auto_trim(val))

            return trimmed_list
       
        elif type(data) is dict:
            trimmed_dict = {}

            for key in data:
                trimmed_dict[key] = self.__auto_trim(data[key])

            return trimmed_dict

        return data






class Required(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, unuse, unuse_):
        if value == None or value == '':
            return 'REQUIRED'

class NotEmpty(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, unuse, unuse_):
        if value == '':
            return 'CANNOT_BE_EMPTY'

class NotEmptyList(object):
    def __init__(self, *args):
        pass
        
    def __call__(self, lst, unuse, unuse_):
        if lst == None or lst == '':
            return 'CANNOT_BE_EMPTY'
        if not isinstance(lst, list):
            return 'WRONG_FORMAT'
        if not lst:
            return 'CANNOT_BE_EMPTY'



import os
import yaml
import unittest

class TestSuite(unittest.TestCase):
    def runTest(self, path, core):
        for root, dirs, files in os.walk(path):
            self._curent_test = ''.join(root.split('/')[-1])
            files_path = ['/'.join([root,file_name]) for file_name in files]
            result = core(self._prepare(files_path), self._curent_test)

            if result:
                self.assertEqual(*result)

    def _prepare(self, files_path):
        data = {}

        for path in files_path:
            name_in_dict = ''.join(path.split('/')[-1]).replace('.json', '')
            with open(path) as f:
                data[name_in_dict] = yaml.load(f)
        
        return data



import Validator

class BuildAliasedRule(object):
    def __init__(self, source, alias):
        if not isinstance(alias, dict):
            raise Exception('Wrong alias format. Dict required')
        if not 'name' in alias:
            raise Exception('Alias name required')
        if not 'rules' in alias:
            raise Exception('Alias rules required')
        if 'error' in alias:
            self._error = alias['error']
        else:
            self._error = None

        self._livr = {'value': alias['rules']}
        source[alias['name']] = self

    def __call__(self, *args):
        self.__rule_builders = args[0]
        self.__validator = Validator.Validator(self._livr)

        self.__validator.register_rules(self.__rule_builders)
        self.__validator.prepare()

        return self._validate

    def _validate(self, value, unuse, output):    
        result = self.__validator.validate({'value': value})

        if not result:
            return self._error or self.__validator.get_errors()['value']
        
        output.append(result['value'])




import os
import sys
PATH_TO_TESTS_INV = '/'.join(str(os.path.abspath(__file__)).split('/')[:-1])
PATH_TO_VALIDATOR = '/'.join(str(os.path.abspath(__file__)).split('/')[:-2])
sys.path.insert(0,PATH_TO_VALIDATOR)
import Validator
from livr_test import TestSuite

def negative_test(data, current_test):
    if not 'rules' in data or not 'input' in data or not 'errors' in data:
        return
    validator = Validator.Validator(data['rules'])
    validator.validate(data['input'])

    """ If need to debug uncoment next code"""
    # standard_errors = data['errors']
    # current_errors = validator.get_errors()
    # if standard_errors == current_errors:
    #   print('Test \'{0}\' is Passed!!'.format(current_test))
    # else:
    #   print('Test \'{0}\' is NOT passed \n Gives:\n\n{}\n\n Must give:\n\n{}\n\n'.format(current_test,
    #                                                                                     current_errors,
    #                                                                                     standard_errors))

    return [data['errors'], validator.get_errors()]

def positive_test(data, current_test):
    if not 'rules' in data or not 'input' in data or not 'output' in data:
            return
    validator = Validator.Validator(data['rules'])
    result = validator.validate(data['input'])

    """ If need to debug uncoment next code"""
    # standart_output = data['output']
    # if result:
    #     if standart_output == result:
    #         print('Test \'{0}\' is Passed!!'.format(current_test))
    #     else:
    #         print('Test \'{0}\' is NOT passed\nGives result:\n\n{1}\n\nMust give:\n\n{2}\n\n'.format(current_test,
    #                                                                                                     result,
    #                                                                                                     standart_output))
    # else:
    #     print('Test \'{0}\' is NOT passed\nGives errors:\n\n{1}\n\n Must give:\n\n{2}\n\n'.format(current_test,
    #                                                                                             validator.get_errors(),
    #                                                                                             standart_output))
    return [data['output'], result]

def aliase_negative_test(data, current_test):

    if not 'rules' in data or not 'input' in data or not 'errors' in data or not 'aliases' in data:
        return
    validator = Validator.Validator(data['rules'])
    
    for alias in data['aliases']:
        validator.register_aliased_rule(alias) 
    validator.validate(data['input'])

    """ If need to debug uncoment next code"""
    # standard_errors = data['errors']
    # current_errors = validator.get_errors()
    # if standard_errors == current_errors:
    #   print('Test \'{0}\' is Passed!!'.format(current_test))
    # else:
    #   print('Test \'{0}\' is NOT passed \n Gives:\n\n{1}\n\n Must give:\n\n{2}\n\n'.format(current_test,
    #                                                                                         current_errors,
    #                                                                                         standard_errors))

    return [data['errors'], validator.get_errors()]

def aliase_positive_test(data, current_test): 
    if not 'rules' in data or not 'input' in data or not 'output' in data or not 'aliases' in data:
        return
    validator = Validator.Validator(data['rules'])
    
    for alias in data['aliases']:
        validator.register_aliased_rule(alias) 
    result = validator.validate(data['input'])

    """ If need to debug uncoment next code"""
    # standart_output = data['output']
    # if result:
    #     if standart_output == result:
    #         print('Test \'{0}\' is Passed!!'.format(current_test))
    #     else:
    #         print('Test \'{0}\' is NOT passed\nGives result:\n\n{1}\n\nMust give:\n\n{2}\n\n'.format(current_test,
    #                                                                                             result,
    #                                                                                             standart_output))
    # else:
    #     print('Test \'{0}\' is NOT passed\nGives errors:\n\n{1}\n\n Must give:\n\n{2}\n\n'.format(current_test,
    #                                                                                         validator.get_errors(),
    #                                                                                         standart_output))

    return [data['output'], result]

if __name__ == "__main__":
    test_engine = TestSuite()

    test_engine.runTest(PATH_TO_TESTS_INV+'/test_suite/positive', positive_test)
    test_engine.runTest(PATH_TO_TESTS_INV+'/test_suite/negative', negative_test)
    test_engine.runTest(PATH_TO_TESTS_INV+'/test_suite/aliases_negative', aliase_negative_test)
    test_engine.runTest(PATH_TO_TESTS_INV+'/test_suite/aliases_positive', aliase_positive_test)
    


import re
ESCAPE_RE = lambda string: string.replace('[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]', "\\$&")

class Trim(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, unuse, output):
        if value == None or value == '' or not isinstance(value, str):
            return

        output.append(value.strip())

class ToLc(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, unuse, output):
        if value == None or value == '' or not isinstance(value, str):
            return

        output.append(value.lower())

class ToUc(object):
    def __init__(self, *args):
        pass
    
    def __call__(self, value, unuse, output):
        if value == None or value == '' or not isinstance(value, str):
            return

        output.append(value.upper())

class Remove(object):
    def __init__(self, *args):
        self._chars = ESCAPE_RE(args[1])

    def __call__(self, value, unuse, output):
        if value == None or value == '' or not isinstance(value, str):
            return

        output.append(re.sub('[{}]'.format(self._chars), '', value))

class LeaveOnly(object):
    def __init__(self, *args):
        self._chars = ESCAPE_RE(args[1])

    def __call__(self, value, unuse, output):
        if value == None or value == '' or not isinstance(value, str):
            return

        output.append(re.sub('[^{}]'.format(self._chars), '', value))



import re

class OneOf(object):
    def __init__(self, *args):
        self._allowed_list = args[1]
        
        # if not isinstance(self._allowed_list, list):
        #     raise Exception('Wrong standard value! List Required')
    
    def __call__(self, value, unuse, unuse_):
        if not value:
            return
        if not value in self._allowed_list:
            return 'NOT_ALLOWED_VALUE'

class MaxLength(object):
    def __init__(self, *args):
        self._max_length = int(args[1])
        # if self._max_length < 1:
        #     raise Exception('Wrong standard value! Only positive value')

    def __call__(self, value, unuse, unuse_):
        if not value:
            return
        if len(value) > self._max_length:
            return 'TOO_LONG'

class MinLength(object):
    def __init__(self, *args):
        self._min_length = args[1]

    def __call__(self, value, unuse, unuse_):
        if not value:
            return
        if len(value) < self._min_length:
            return 'TOO_SHORT'

class EqualLength(object):
    def __init__(self, *args):
        self._length = args[1]

    def __call__(self, value, unuse, unuse_):
        if not value:
            return
        if len(value) > self._length:
            return 'TOO_LONG'
        if len(value) < self._length:
            return 'TOO_SHORT'

class BetweenLength(object):
    def __init__(self, *args):
        self._min_length = args[1]
        self._max_length = args[2]
        # if self._max_length < 1:
        #     raise Exception('Wrong standard value! Only positive value')

    def __call__(self, value, unuse, unuse_):
        if not value:
            return
        if len(value) > self._max_length:
            return 'TOO_LONG'
        if len(value) < self._min_length:
            return 'TOO_SHORT'

class Like(object):
    def __init__(self, *args):
        self._sample = args[1]

    def __call__(self, value, unuse, unuse_):
        if not value:
            return
        if not re.match(self._sample, str(value)):
            return 'WRONG_FORMAT'




import sys
import os
ROOT_PATH = '/'.join(str(os.path.abspath(__file__)).split('/')[:-2])
sys.path.insert(0,ROOT_PATH)

class NestedObject(object):
    def __init__(self, *args):
            import Validator
            self._livr = args[1]
            self._rule_builders = args[0]
            self._validator = Validator.Validator(self._livr)

            self._validator.register_rules(self._rule_builders)
            self._validator.prepare()

    def __call__(self, nested_obj, unuse, output):
        if not nested_obj or nested_obj == 0:
            return
        if not isinstance(nested_obj, dict):
            return 'FORMAT_ERROR'

        result = self._validator.validate(nested_obj)

        if not result:  
            return self._validator.get_errors()

        output.append(result) 

class ListOf(object):
    def __init__(self, *args):
        import Validator
        self._livr = args[1]
        self._rule_builders = args[0]
        self._validator = Validator.Validator({'field':self._livr})

        self._validator.register_rules(self._rule_builders)
        self._validator.prepare()

    def __call__(self, values, unuse, output):   
        if not values or values == 0:
            return
        if not isinstance(values, dict) and not isinstance(values, list):
            return 'FORMAT_ERROR'
        return self._check_validation(values, output)

    def _check_validation(self, values, output):
        results = []
        errors = []
        error_flag = False

        for val in values:
            result = self._validator.validate({'field': val})
            
            if result:
                results.append(result['field'])
                errors.append(None)
            else:
                error_flag = True
                errors.append(self._validator.get_errors()['field'])

        if error_flag:
            return errors

        output.append(results)

class ListOfObjects(object):
    def __init__(self, *args):
        import Validator
        self._livr = args[1]
        self._rule_builders = args[0]
        self._validator = Validator.Validator(self._livr)

        self._validator.register_rules(self._rule_builders)
        self._validator.prepare()

    def __call__(self, objects, unuse, output):
        if objects == None or objects == '':
            return
        if not isinstance(objects, dict) and not isinstance(objects, list):
            return 'FORMAT_ERROR'   

        return self._check_validation(objects, output)

    def _check_validation(self, objects, output):
        results = []
        errors = []
        error_flag = False

        for obj in objects:
            if not isinstance(obj, dict):
                errors.append('FORMAT_ERROR')
                continue

            result = self._validator.validate(obj)
            
            if result:
                results.append(result)
                errors.append(None)
            else:
                error_flag = True
                errors.append(self._validator.get_errors())

        if error_flag:
            return errors

        output.append(results)

class ListOfDiferentObjects(object):
    def __init__(self, *args):
        import Validator
        self._selector_fields = args[1]
        self._livrs = args[2]
        self._rule_builders = args[0]
        self._validators = {}


        for selector, livr in self._livrs.items():
            validator = Validator.Validator(livr)
            
            validator.register_rules(self._rule_builders)
            validator.prepare()
            self._validators[selector] = validator

    def __call__(self, objects, unuse, output):
        results = []
        errors = []
        error_flag = False

        if not objects or objects == 0:
            return
        if not isinstance(objects, list):
            return 'FORMAT_ERROR'

        for obj in objects:
            if not isinstance(obj, dict) and not self._selector_fields in obj:
                errors.append("FORMAT_ERROR")
                continue
            if not obj[self._selector_fields] in self._validators:
                errors.append("FORMAT_ERROR")
                continue

            validator = self._validators[obj[self._selector_fields]]
            result = validator.validate(obj)

            if result:
                results.append(result)
                errors.append(None)
            else:
                error_flag = True
                errors.append(validator.get_errors())
        
        if error_flag:
            return errors

        output.append(results)





import re
import datetime

class Email(object):
    _reg_exp_mail = '^(?:[a-z0-9!#$%&\'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&\'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])+$'
    def __init__(self, *args):
        pass
    
    def __call__(self, email, unuse, unuse_):
        if not isinstance(email, str):
            return 'FORMAT_ERROR'            
        if email == '':
            return
        if not re.match(self._reg_exp_mail, email):
            return 'WRONG_EMAIL'

class EqualToField(object):
    def __init__(self, *args):
        self._field = args[1]

    def __call__(self, value, params, unuse):
        if not value or value == 0:
            return
        if value != params[self._field]:
            return 'FIELDS_NOT_EQUAL'

class Url(object):
    _url_re = re.compile(
        r'^(?:http|https)s?://'
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
        r'localhost|'
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
        r'(?::\d+)?'
        r'(?:/?|[/?]\S+)$', re.I)
    
    def __init__(self, *args):
        pass

    def __call__(self, value, params, unuse):
        if not value or value == 0:
            return
        if len(value) < 2083 and self._url_re.match(value):
            return

        return 'WRONG_URL'

class IsoDate(object):
    def __init__(self, *args):
        pass

    def __call__(self, value, params, unuse):
        if not value or value == 0:
            return

        try:
            datetime.datetime.strptime(value, '%Y-%m-%d')
        except ValueError:
            return 'WRONG_DATE'



from distutils.core import setup

with open('README.txt') as readme:
    setup(
        name = 'LIVR',
        version = '0.4',
        author = 'Viktor Turstkiy, Ihor Kolosha',
        packages = ['LIVR', 'LIVR.test', 'LIVR.Rules'],
        license='look into README',
        description='LIVR validator.',
        long_description=readme.read(),
    )